#!/usr/bin/env python3
#-*-coding: utf-8-*-
"""
tkentrycomplete.py

A tkinter widget that features autocompletion.

Created by Mitja Martini on 2008-11-29.
Changed for use in apodixispro from Konstas Marmatakis
"""
import tkinter
from tkinter import ttk

__version__ = "1.0.1"

tkinter_umlauts=['odiaeresis', 'adiaeresis', 'udiaeresis', 'Odiaeresis', 'Adiaeresis', 'Udiaeresis', 'ssharp']

tkinter_greek = ['GREEK SMALL LETTER ALPHA', 'GREEK SMALL LETTER BETA', 'GREEK SMALL LETTER GAMMA', 'GREEK SMALL LETTER DELTA',
                 'GREEK SMALL LETTER EPSILON', 'GREEK SMALL LETTER ZETA', 'GREEK SMALL LETTER ETA', 'GREEK SMALL LETTER THETA',
                 'GREEK SMALL LETTER IOTA', 'GREEK SMALL LETTER KAPPA', 'GREEK SMALL LETTER LAMDA', 'GREEK SMALL LETTER MU',
                 'GREEK SMALL LETTER NU', 'GREEK SMALL LETTER XI', 'GREEK SMALL LETTER OMICRON', 'GREEK SMALL LETTER PI',
                 'GREEK SMALL LETTER RHO', 'GREEK SMALL LETTER SIGMA', 'GREEK SMALL LETTER TAU', 'GREEK SMALL LETTER UPSILON',
                 'GREEK SMALL LETTER PHI', 'GREEK SMALL LETTER CHI', 'GREEK SMALL LETTER PSI', 'GREEK SMALL LETTER OMEGA',
                 'GREEK SMALL LETTER ALPHA WITH TONOS', 'GREEK SMALL LETTER EPSILON WITH TONOS', 'GREEK SMALL LETTER ETA WITH TONOS',
                 'GREEK SMALL LETTER IOTA WITH TONOS', 'GREEK SMALL LETTER OMICRON WITH TONOS', 'GREEK SMALL LETTER UPSILON WITH TONOS',
                 'GREEK SMALL LETTER OMEGA WITH TONOS', 'GREEK SMALL LETTER IOTA WITH DIALYTIKA AND TONOS',
                 'GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND TONOS', 'GREEK CAPITAL LETTER ALPHA', 'GREEK CAPITAL LETTER BETA',
                 'GREEK CAPITAL LETTER GAMMA', 'GREEK CAPITAL LETTER DELTA', 'GREEK CAPITAL LETTER EPSILON', 'GREEK CAPITAL LETTER ZETA',
                 'GREEK CAPITAL LETTER ETA', 'GREEK CAPITAL LETTER THETA', 'GREEK CAPITAL LETTER IOTA', 'GREEK CAPITAL LETTER KAPPA',
                 'GREEK CAPITAL LETTER LAMDA', 'GREEK CAPITAL LETTER MU', 'GREEK CAPITAL LETTER NU', 'GREEK CAPITAL LETTER XI',
                 'GREEK CAPITAL LETTER OMICRON', 'GREEK CAPITAL LETTER PI', 'GREEK CAPITAL LETTER RHO', 'GREEK CAPITAL LETTER SIGMA',
                 'GREEK CAPITAL LETTER TAU', 'GREEK CAPITAL LETTER UPSILON', 'GREEK CAPITAL LETTER PHI', 'GREEK CAPITAL LETTER CHI',
                 'GREEK CAPITAL LETTER PSI', 'GREEK CAPITAL LETTER OMEGA', 'GREEK SMALL LETTER FINAL SIGMA', 'GREEK CAPITAL LETTER ALPHA WITH TONOS',
                 'GREEK CAPITAL LETTER EPSILON WITH TONOS', 'GREEK CAPITAL LETTER IOTA WITH TONOS', 'GREEK CAPITAL LETTER ETA WITH TONOS',
                 'GREEK CAPITAL LETTER OMICRON WITH TONOS', 'GREEK CAPITAL LETTER UPSILON WITH TONOS', 'GREEK CAPITAL LETTER OMEGA WITH TONOS']


class AutocompleteEntry(ttk.Entry):
        """
        Subclass of Tkinter.Entry that features autocompletion.
        
        To enable autocompletion use set_completion_list(list) to define 
        a list of possible strings to hit.
        To cycle through hits use down and up arrow keys.
        """
        def set_completion_list(self, completion_list):
                self._completion_list = completion_list
                self._hits = []
                self._hit_index = 0
                self.position = 0
                self.bind('<KeyRelease>', self.handle_keyrelease)

        def autocomplete(self, delta=0):
                """autocomplete the Entry, delta may be 0/1/-1 to cycle through possible hits"""
                if delta: # need to delete selection otherwise we would fix the current position
                        self.delete(self.position, tkinter.END)
                else: # set position to end so selection starts where textentry ended
                        self.position = len(self.get())
                # collect hits
                _hits = []
                for element in self._completion_list:
                        if element.startswith(self.get().lower()):
                                _hits.append(element)
                # if we have a new hit list, keep this in mind
                if _hits != self._hits:
                        self._hit_index = 0
                        self._hits=_hits
                # only allow cycling if we are in a known hit list
                if _hits == self._hits and self._hits:
                        self._hit_index = (self._hit_index + delta) % len(self._hits)
                # now finally perform the auto completion
                if self._hits:
                        self.delete(0,tkinter.END)
                        self.insert(0,self._hits[self._hit_index])
                        self.select_range(self.position,tkinter.END)

        def handle_keyrelease(self, event):
                """event handler for the keyrelease event on this widget"""
                if event.keysym == "BackSpace":
                        self.delete(self.index(tkinter.INSERT), tkinter.END)
                        self.position = self.index(tkinter.END)
                if event.keysym == "Left":
                        if self.position < self.index(tkinter.END): # delete the selection
                                self.delete(self.position, tkinter.END)
                        else:
                                self.position = self.position-1 # delete one character
                                self.delete(self.position, tkinter.END)
                if event.keysym == "Right":
                        self.position = self.index(tkinter.END) # go to end (no selection)
                if event.keysym == "Down":
                        self.autocomplete(1) # cycle to next hit
                if event.keysym == "Up":
                        self.autocomplete(-1) # cycle to previous hit
                # perform normal autocomplete if event is a single key or an umlaut
                if len(event.keysym) == 1 or event.keysym in tkinter_umlauts:
                        self.autocomplete()

def test(test_list):
        """Run a mini application to test the AutocompleteEntry Widget."""
        root = tkinter.Tk(className=' AutocompleteEntry demo')
        entry = AutocompleteEntry(root)
        entry.set_completion_list(test_list)
        entry.pack()
        entry.focus_set()
        root.mainloop()

if __name__ == '__main__':
##        test_list = ('test', 'type', 'true', 'tree', 'tölz', '028555248', 'Κακός λύκος', 'γάιδαρος')
        import sqlite3
        con = sqlite3.connect('../databak/apodixispro.db')
        cur = con.cursor()
        qry = "SELECT afm FROM epixirisis ORDER BY afm"

        cur.execute(qry)
        
        test_list = []
        
        for r in cur.fetchall():
                test_list.append(r[0])
        con.close()

        print("Type a 't' to test the AutocompleteEntry widget.")
        print("Will use AutocompleteEntry.set_completion_list(%s)" % str(test_list))
        print("Try also the backspace key and the arrow keys.")
        test(test_list)
